import { Test, TestingModule } from '@nestjs/testing';
import { getRepositoryToken } from '@nestjs/typeorm';
import { AlbumService } from './album.service';
import { Album } from './album.entity';
import { CreateAlbumDto } from './dto/create-album.dto';
import { UpdateAlbumDto } from './dto/update-album.dto';
import { DeleteResult, Repository } from 'typeorm';

const mockAlbum = new Album();
const mockCreateAlbumDto = new CreateAlbumDto();
const mockUpdateAlbumDto = new UpdateAlbumDto();
const mockDeleteResult = new DeleteResult();

describe('AlbumService', () => {
  let service: AlbumService;
  let repository: Repository<Album>;

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        AlbumService,
        {
          provide: getRepositoryToken(Album),
          useValue: {
            find: jest.fn().mockResolvedValue([mockAlbum]),
            findOne: jest.fn().mockResolvedValue(mockAlbum),
            findOneBy: jest.fn().mockResolvedValue({ id: 1, title: 'some album', artist: 'some artist', genre: 'some genre' }),
            create: jest.fn().mockResolvedValue(mockAlbum),
            save: jest.fn().mockResolvedValue({ id: 1, title: 'some album', artist: 'some artist', genre: 'some genre' }),
            update: jest.fn().mockResolvedValue({ id: 1, title: 'some album', artist: 'some artist', genre: 'some genre' }),
            delete: jest.fn().mockResolvedValue(mockDeleteResult),
          },
        },
      ]
    }).compile();

    service = module.get<AlbumService>(AlbumService);
    repository = module.get<Repository<Album>>(getRepositoryToken(Album));
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('findAll', () => {
    it('should return an array of Albums', async () => {
      expect(await service.findAll()).toStrictEqual([mockAlbum]);
    })
  })

  describe('findOneById', () => {
    it('should return an Album', async () => {
      expect(await service.findOneById(1)).toStrictEqual(mockAlbum);
    })
  })

  describe('create', () => {
    it('should return the Album', async () => {
      expect(await service.create(mockCreateAlbumDto)).toStrictEqual({ id: 1, title: 'some album', artist: 'some artist', genre: 'some genre' });
    })
  })

  describe('update', () => {
    it('should return the Album', async () => {
      mockUpdateAlbumDto.id = 1;
      expect(await service.update(1, mockUpdateAlbumDto)).toStrictEqual({ id: 1, title: 'some album', artist: 'some artist', genre: 'some genre' });
    })
  })

  describe('remove', () => {
    it('should return a DeleteResult', async () => {
      expect(await service.remove(1)).toStrictEqual(mockDeleteResult);
    })
  })
});
